/* -------------------------------------------------------------------------- *
 * file:         scanprocess.c                                                *
 * purpose:      start the scan and retrieve the scanner output               *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <cgic.h>

#include "nessuswc.h"

int cgiMain() {

   static char 	title[] = "Scanning Progress";
   char		nessusd_ip[16];
   int		nessusd_port;
   char		nessusd_user[81];
   char		nessusd_pass[81];
   char		nessusd_cert[81];
   char		nessusd_encr[81];
   char		target_ip[16];
   char		target_mask[16];
   char		formbuf[2];
   int		plugs_counter = 0;
   int		prefs_counter = 0;
   int		rules_counter = 0;
   int		error_counter = 0;
   int		famly_enabled = 0;
   int		plugs_enabled = 0;
   int		enabled_counter = 0;
   char		*session_id;
   SSL		*ssl;
   int 		i = 0, j = 0, k = 0;

/* -------------------------------------------------------------------------- *
 * check if we got all information to make a nessus server connection         *
 * ---------------------------------------------------------------------------*/

  if ( cgiFormString("nessusd-ip", nessusd_ip, sizeof(nessusd_ip))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus server IP address.");

  if ( cgiFormInteger("nessusd-port", &nessusd_port, NESSUSD_PORT)
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus server port number.");

  if ( nessusd_port <= 0 || nessusd_port > 65535 )
    int_error("Error Nessus server port number not in a valid port range.");

  if ( cgiFormString("nessusd-encr", nessusd_encr, sizeof(nessusd_encr))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus server protocol encryption type.");

  if ( cgiFormString("nessusd-user", nessusd_user, sizeof(nessusd_user))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus remote user name.");

  if ( cgiFormString("nessusd-pass", nessusd_pass, sizeof(nessusd_pass))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus remote user password.");

  if ( cgiFormString("nessusd-cert", nessusd_cert, sizeof(nessusd_cert))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus client certificate name.");
  
  if ( cgiFormString("t-ip", target_ip, sizeof(nessusd_cert))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus client scan target IP address.");
  
  if ( cgiFormString("t-mask", target_mask, sizeof(nessusd_cert))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus client scan target netmask.");
  
/* -------------------------------------------------------------------------- *
 * make a TCP connection to the Nessus Server.                                *
 * ---------------------------------------------------------------------------*/

  ssl = nessus_connect(nessusd_ip, nessusd_port, nessusd_encr, nessusd_cert);

/* -------------------------------------------------------------------------- *
 * Handle the login to the Nessus Server.                                     *
 * ---------------------------------------------------------------------------*/

  nessus_login(ssl, nessusd_user, nessusd_pass);

/* -------------------------------------------------------------------------- *
 * Get the list of plugins from the Nessus Server and                         *
 * create the list of categories.                                             *
 * ---------------------------------------------------------------------------*/

  plugs_counter = nessus_getplugs(ssl);
  if (plugs_counter == 0)
     int_error("Could not receive plugins from Nessus server.");

  nessus_getgroups(plugs_counter);

/* -------------------------------------------------------------------------- *
 * Get the preferences list from the Nessus Server.                           *
 * ---------------------------------------------------------------------------*/

  prefs_counter = nessus_getprefs(ssl);
  if (prefs_counter == 0)
     int_error("Could not receive preferences from Nessus server.");

/* -------------------------------------------------------------------------- *
 * Get the rules list from the Nessus Server.                                 *
 * ---------------------------------------------------------------------------*/

  rules_counter = nessus_getrules(ssl);
  /* it isn't unusual to have no rules so the count can be zero. */

/* -------------------------------------------------------------------------- *
 * Here we process the prefs and plugins before we can return the list        *
 * ---------------------------------------------------------------------------*/

  if (cgiFormString("template", templatefilestr, sizeof(templatefilestr))
                                                         == cgiFormNotFound) {
    for(i=0; i<famly_counter; i++) {
      if (cgiFormString(famlylist[i].name, formbuf, sizeof(formbuf))
                                                         != cgiFormNotFound) {
        famlylist[i].enabled = 1;
        famly_enabled++;
      }
    }
    if (famly_enabled == 0)
      int_error("No plugin family had been selected. Choose at least one.");

  /* ------------------------------------------------------------------------ *
   * Send the preferences back to the Nessus Server.                          *
   * -------------------------------------------------------------------------*/

    nessus_setprefs(ssl, prefs_counter);

  /* ------------------------------------------------------------------------ *
   * Send the enabled plugin list to the Nessus Server.                       *
   * -------------------------------------------------------------------------*/

    plugs_enabled = nessus_setplugs(ssl);
    if (plugs_enabled == 0)
      int_error("Sent no plugin ID's to the server.");

  } else {

    /* load the template information */
    get_template_name(templatefilestr);
    get_template_prefs(templatefilestr);
    get_template_plugs(templatefilestr);

    if(template_plugs_enabled == 0)
      int_error("No plugin has been enabled. Enable at least one in template.");

    /* update the preference list */
    for (i=0; i<prefs_counter; i++) {
      for (j=0; j<template_prefs_all; j++) {
        if(strcmp(prefslist_ptr[i]->name, template_pref_list[j].name) == 0)
          if(strcmp(prefslist_ptr[i]->value, template_pref_list[j].value) != 0)
            strncpy(prefslist_ptr[i]->value, template_pref_list[j].value,
                    sizeof(prefslist_ptr[i]->value));
      }
    }

    /* count the enabled families */
    for(i=0; i<famly_counter; i++) {

    /* check for each family, how many of the enabled template    *
     * plugins match the plugin list for this family and increase *
     * the enabled_counter with each hit.                         */

       enabled_counter = 0;
       for(j=0; j<famlylist[i].plugscount; j++) {
         for(k=0; k<template_plugs_all; k++) {
           if( strcmp(famlylist[i].plugs_ptr[j]->id,
               template_plug_list[k].id) == 0
               &&
               template_plug_list[k].enabled == 1 )
             enabled_counter++;
         }
       }
       if(enabled_counter > 0) famly_enabled++;
     } 
  /* ------------------------------------------------------------------------ *
   * Send the preferences back to the Nessus Server.                          *
   * -------------------------------------------------------------------------*/

    nessus_setprefs(ssl, prefs_counter);

  /* ------------------------------------------------------------------------ *
   * Send the enabled plugin list to the Nessus Server.                       *
   * -------------------------------------------------------------------------*/

    plugs_enabled = template_setplugs(ssl);
    if (plugs_enabled == 0)
      int_error("Sent no plugin ID's to the server.");
  } 

  /* ------------------------------------------------------------------------ *
   * Send the target IP list to the Nessus Server.                            *
   * -------------------------------------------------------------------------*/

    nessus_target(ssl, target_ip, target_mask);

  /* ------------------------------------------------------------------------ *
   * Receive the scan config errors from the Nessus Server. Should be zero.   *
   * -------------------------------------------------------------------------*/

    error_counter = nessus_geterrs(ssl);
    if (error_counter != 0)
     int_error("Server signals errors in client config transfer.");

/* -------------------------------------------------------------------------- *
 * start the initial cgi and update output                                    *
 * ---------------------------------------------------------------------------*/

   session_id = create_session_id(target_ip);
   
   /* start writing the update popup window data so we are ready to display */
   write_hostupdates(session_id, target_ip, "start", "0");

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

   pagehead(title, session_id, cgiOut);

   fprintf(cgiOut, "<p><center>");
   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=2>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Initiating new Nessus Scan");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF");
   fprintf(cgiOut, " width=180>");
   fprintf(cgiOut, "Selected Template:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=\"#FFFFFF\" align=\"center\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
   if(strlen(template_name) == 0)
     strncpy(template_name, "none", sizeof(template_name));
   fprintf(cgiOut, "%s", template_name);
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Selected Families:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=\"#FFFFFF\" align=\"center\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
   fprintf(cgiOut, "%d", famly_enabled);
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Enabled Plugins:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=\"#FFFFFF\" align=\"center\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
   fprintf(cgiOut, "%d", plugs_enabled);
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Selected Target:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=\"#FFFFFF\" align=\"center\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
   fprintf(cgiOut, "%s", target_ip);
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Scanner Progress:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=\"#FFFFFF\" align=\"center\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
   fprintf(cgiOut, "<img src=../images/progressbar.gif>");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Detailed Updates:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=\"#FFFFFF\" align=\"center\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
   fprintf(cgiOut, "<input type=button OnClick=\"return ");
   fprintf(cgiOut, "window.open(\'%s/updates-%s.htm\',",RESULTS_DIR,session_id);
   fprintf(cgiOut, " \'progress_update\',");
   fprintf(cgiOut, " 'width=600,height=150,status=no,scrollbars=no');\"\n");
   fprintf(cgiOut, " value=\"Show Scan Progress Details\">");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=2>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Total: %d Plugins", plugs_counter );
   fprintf(cgiOut, " in %d Families.", famly_counter );
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "</table>\n");

   fprintf(cgiOut, "<p><center>");
   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=2>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "NessusWC Client");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF");
   fprintf(cgiOut, " width=180>");
   fprintf(cgiOut, "Version:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td align=\"center\"bgcolor=\"#FFFFFF\">");
   fprintf(cgiOut, "%s\n", CLIENT_SW_VERSION);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>\n");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Copyright:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td align=\"center\"bgcolor=\"#FFFFFF\">");
   fprintf(cgiOut, "%s\n", CLIENT_COPYRIGHTS);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Status:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td align=\"center\"bgcolor=\"#FFFFFF\">");
   fprintf(cgiOut, "<b>%s</b> login to server <b>%s</b> successful.",
                    nessusd_user, nessusd_ip);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=2>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "End");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "</table>\n");

/* -------------------------------------------------------------------------- *
 * end the html output                                                        *
 * ---------------------------------------------------------------------------*/

  fprintf(cgiOut, "</BODY></HTML>\n");
  fflush(cgiOut);

/* -------------------------------------------------------------------------- *
 * Receive the server status updates and results.                             *
 * ---------------------------------------------------------------------------*/

  nessus_getstats(ssl, session_id);
  write_hostresults(target_ip, nessusd_ip, session_id);

  fclose(cgiOut);
  return(0);
}
